#!/bin/bash

### 人脸图片流负载均衡状态检查
FacePicBalanceStatusCheck()
{
    pic_balance_strategy_info=` iamcfg list | grep pic-balance-strategy `
    # 55版本之后取消pic-balance-strategy配置项，只支持负载均衡模式
    if [ -z "${pic_balance_strategy_info}" ]
    then
        ResultTxtOutput "INFO" "人脸图片流默认为负载均衡状态"
        return
    fi
    
    pic_balance_strategy=$(echo ${pic_balance_strategy_info} | awk -F " " '{print $3}')
    if [ 1 -eq $pic_balance_strategy ]
    then
        ResultTxtOutput "INFO" "人脸图片流负载均衡状态已开启"
    else
        ResultTxtOutput "WARNING" "人脸图片流负载均衡状态未开启，建议开启"
    fi
}

### 防火墙状态检查
FirewalldStatusCheck()
{
    firewalld_status_info=` systemctl status firewalld | grep Active `
    firewalld_status=$(echo ${firewalld_status_info} | awk -F " " '{print $2}')
    
    # 防火墙状态获取为空
    if [ -z "${firewalld_status}" ]
    then
        ResultTxtOutput "WARNING" "防火墙状态检查失败，请手动检查"
        return
    fi
    
    if [ "inactive" == "${firewalld_status}" ]
    then
        ResultTxtOutput "INFO" "iam防火墙已关闭"
    else
        ResultTxtOutput "WARNING" "iam防火墙未关闭，建议关闭"
    fi
}

### 芯片运行状态检查
ChipRunStatusCheck()
{
    # 获取引擎数量
    # engine_count=` SelectDB "sqlite" "SELECT COUNT(*) FROM ia_engine;" `
    engine_count=` sqlite3 /opt/wise/ia12k/db/ias.db "SELECT COUNT(*) FROM ia_engine;" `
    # 获取iam的进程号
    iam_pid_old=` pidof iam `
    if [ -z $iam_pid_old ]
    then
        ResultTxtOutput "WARNING" "iam进程号获取失败，芯片运行状态检查失败，请手动检查"
        return
    fi
    cmd=` kill -59 ${iam_pid_old} `
    # 当前时间
    now_timestamp=$(date +%s)
    now_time=$(date -d @"$now_timestamp" +"%H:%M:%S")
    # 当前时间往前推一秒
    previous_one_timestamp=$((now_timestamp - 1))
    previous_one_time=$(date -d @"$previous_one_timestamp" +"%H:%M:%S")
    # 当前时间往后推一秒
    after_one_timestamp=$((now_timestamp + 1))
    after_one_time=$(date -d @"$after_one_timestamp" +"%H:%M:%S")
    # 当前时间往后推两秒
    after_two_timestamp=$((now_timestamp + 2))
    after_two_time=$(date -d @"$after_two_timestamp" +"%H:%M:%S")
    # 等待两秒，两秒后收日志
    sleep 2
    iam_log=` cat /var/log/iamlog/iam00.log | grep -E 'engine.*vesion.*runStatus.*authStatus.*businessType.*workMode.*taskNum' | grep -E "${now_time}|${previous_one_time}|${after_one_time}|${after_two_time}"`
    iam_pid_new=` pidof iam `
    # 如果kill -59 导致进程重启，则不在执行该命令
    if [ ${iam_pid_old} != ${iam_pid_new} ]
    then
        LOG "当前版本不支持kill -59命令，iam pid：${iam_pid_new}  ${iam_pid_old}" 
        ResultTxtOutput "WARNING" "芯片运行状态检查失败，请手动检查"
        return
    fi
    # 如果在iam日志中获取到的芯片数量与数据库中的已知，则不进行第二次获取
    IFS=$'\n' read -d '' -ra arr_iam_log_info <<< "$iam_log"
    if [ ${#arr_iam_log_info[@]} -ne ${engine_count} ]
    then
        # 如果收集日志中的芯片数量与sqlite数据库中的芯片数量不一致，则需要进行二次收集
        cmd=` kill -59 ${iam_pid_old} `
        # 当前时间
        now_timestamp=$(date +%s)
        now_time=$(date -d @"$now_timestamp" +"%H:%M:%S")
        # 当前时间往前推一秒
        previous_one_timestamp=$((now_timestamp - 1))
        previous_one_time=$(date -d @"$previous_one_timestamp" +"%H:%M:%S")
        # 当前时间往后推一秒
        after_one_timestamp=$((now_timestamp + 1))
        after_one_time=$(date -d @"$after_one_timestamp" +"%H:%M:%S")
        # 当前时间往后推两秒
        after_two_timestamp=$((now_timestamp + 2))
        after_two_time=$(date -d @"$after_two_timestamp" +"%H:%M:%S")
        # 等待两秒，两秒后收日志
        sleep 2
        iam_log=` cat /var/log/iamlog/iam00.log | grep -E 'engine.*vesion.*runStatus.*authStatus.*businessType.*workMode.*taskNum' | grep -E "${now_time}|${previous_one_time}|${after_one_time}|${after_two_time}"`
        IFS=$'\n' read -d '' -ra arr_iam_log_info <<< "$iam_log"
        if [ ${#arr_iam_log_info[@]} -nt ${engine_count} ]
        then
            # 第二次收集的信息仍对不齐，则终止该项检查
            ResultTxtOutput "WARNING" "芯片运行状态检查失败，请手动检查"
            return
        fi
    fi
    error_message=""
    warning_message=""
    for arr_iam_log_info_line in "${arr_iam_log_info[@]}"
	do
        # 芯片索引
        chip_index=""
        # 运行状态
        run_status=""
        # 授权状态
        auth_status=""
        
        # 跳过BW
        if [[ ! "$arr_iam_log_info_line" == *"#"* ]]
        then
            continue
        fi
        
        # 解析芯片号
        iae_ip=$(echo ${arr_iam_log_info_line} | awk -F "#" '{print $2}')
        chip_num=$(echo ${arr_iam_log_info_line} | awk -F "#" '{print $4}')
        chip_index="${iae_ip}#${chip_num}"
        chip_index=$(echo ${chip_index} | awk -F "," '{print $1}')
        
        # 解析运行状态
        run_status_info=$(echo ${arr_iam_log_info_line} | awk -F "," '{print $3}')
        run_status=$(echo ${run_status_info} | awk -F ":" '{print $2}')
        
        # 解析授权状态
        auth_status_info=$(echo ${arr_iam_log_info_line} | awk -F "," '{print $4}')
        auth_status=$(echo ${auth_status_info} | awk -F ":" '{print $2}')
        
        case ${run_status} in
            1)
                # 未注册
                error_message+="${chip_index}芯片未注册。"
                ;;
            2)
                # 离线
                error_message+="${chip_index}芯片离线。"
                ;;
            3)
                # 在线运行
                if [ ${auth_status} -ne 2 ]
                then
                    error_message+="${chip_index}芯片未授权。"
                fi
                ;;
            4)
                # 在线停止
                error_message+="${chip_index}芯片停止。"
                ;;
            5)
                # 在线空闲
                warning_message+="${chip_index}芯片空闲。"
                ;;
            *)
                # 未知状态
                warning_message+="${chip_index}芯片运行状态未知。"
                ;;
        esac
    done
    
    if [ -n "${error_message}" ]
    then
        ResultTxtOutput "ERROR" "${error_message}"
    fi
        
    if [ -n "${warning_message}" ]
    then
        ResultTxtOutput "WARNING" "${warning_message}"
    fi
        
    if [ -z ${error_message} ] && [ -z ${warning_message} ]
    then
        ResultTxtOutput "INFO" "芯片运行及授权状态正常"
    fi
}

### 静态库导库平台检查
StaticFaceEngineTypeCheck()
{
    LOG 1 "开始检查静态导库平台配置检查"
	local cmd_statisface_enginetype="iamcfg list -d iam -g Global -i staticface-enginetype  | grep staticface-enginetype"
    local iamcfg_statisface_enginetype=` iamcfg list -d iam -g Global -i staticface-enginetype  | grep staticface-enginetype `
	RecordCommndInfo "iam" "${cmd_statisface_enginetype}" "${iamcfg_statisface_enginetype}"
	local statisface_enginetype=$(echo ${iamcfg_statisface_enginetype} | awk -F " " '{print $3}')
	if [ "${statisface_enginetype}" == "7" ]
	then
	    LOG 1 "已配置爱芯静态导库"
		local iae_dispose_chip_num=` SelectDB "sqlite" "SELECT COUNT(*) FROM ia_engine WHERE type=18 AND ((business_type>=300 AND business_type <= 305) OR (business_type>=400 and business_type<=404) OR (business_type>=900 and  business_type<=904))" `
		if [ ${iae_dispose_chip_num} -le 0 ]
		then
		    ResultTxtOutput "ERROR" "没有分配可以用于爱芯静态导库的芯片资源"
		else
		    ResultTxtOutput "INFO" "爱芯静态导库环境正常"
		fi
	elif [ "${statisface_enginetype}" == "4" ]
	then
	    LOG 1 "已配置寒武纪静态导库"
		local iae_dispose_chip_num=` SelectDB "sqlite" "SELECT COUNT(*) FROM ia_engine WHERE type=17 AND ((business_type>=300 AND business_type <= 305) OR (business_type>=400 and business_type<=404) OR (business_type>=900 and  business_type<=904))" `
		if [ ${iae_dispose_chip_num} -le 0 ]
		then
		    ResultTxtOutput "ERROR" "没有分配可以用于寒武纪静态导库的芯片资源"
		else
		    ResultTxtOutput "INFO" "寒武纪静态导库环境正常"
		fi
	elif [ "${statisface_enginetype}" == "3" ]
    then
	    LOG 1 "已配置GPU静态导库"
		local iae_dispose_chip_num=` SelectDB "sqlite" "SELECT COUNT(*) FROM ia_engine WHERE type=13 AND ((business_type>=300 AND business_type <= 305) OR (business_type>=400 and business_type<=404) OR (business_type>=900 and  business_type<=904))" `
		if [ ${iae_dispose_chip_num} -le 0 ]
		then
		    ResultTxtOutput "ERROR" "没有分配可以用于GPU静态导库的芯片资源"
		else
		    ResultTxtOutput "INFO" "GPU静态导库环境正常"
		fi
	elif [ "${statisface_enginetype}" == "2" ]
	then
	    LOG 1 "已配置昆仑3静态导库"
		local iae_dispose_chip_num=` SelectDB "sqlite" "SELECT COUNT(*) FROM ia_engine WHERE type=11 OR type=12 AND ((business_type>=300 AND business_type <= 305) OR (business_type>=400 and business_type<=404) OR (business_type>=900 and  business_type<=904))" `
		if [ ${iae_dispose_chip_num} -le 0 ]
		then
		    ResultTxtOutput "ERROR" "没有分配可以用于昆仑3静态导库的芯片资源"
		else
		    ResultTxtOutput "INFO" "昆仑3静态导库环境正常"
		fi
	elif [ "${statisface_enginetype}" == "1" ]
	then
	    LOG 1 "已配置昆仑2静态导库"
		local iae_dispose_chip_num=` SelectDB "sqlite" "SELECT COUNT(*) FROM ia_engine WHERE type=2 OR type=5 OR type=7 OR type=8 OR type=9 AND ((business_type>=300 AND business_type <= 305) OR (business_type>=400 and business_type<=404) OR (business_type>=900 and  business_type<=904))" `
		if [ ${iae_dispose_chip_num} -le 0 ]
		then
		    ResultTxtOutput "ERROR" "没有分配可以用于昆仑2静态导库的芯片资源"
		else
		    ResultTxtOutput "INFO" "昆仑2静态导库环境正常"
		fi
	else
	    ResultTxtOutput "ERROR" "不支持的静态导库平台：${statisface_enginetype}" 
		
	fi
}

### 布控组网检查
DisposeNetCheck()
{
    LOG 1 "开始布控组网检查"
	local cmd_is_dispose="iamcfg list -d iam -g Dispose -i is-dispose | grep is-dispose"
    local iamcfg_is_dispose=` iamcfg list -d iam -g Dispose -i is-dispose | grep is-dispose `
	RecordCommndInfo "iam" "${cmd_is_dispose}" "${iamcfg_is_dispose}"
	local is_dispose=$(echo ${iamcfg_is_dispose} | awk -F " " '{print $3}')
	if [ "${is_dispose}" == "True" ]
	then
	    # 比对服务IP配置检查
		local cmd_dispose_address="iamcfg list -d iam -g Dispose -i dispose-address | grep dispose-address"
	    local iamcfg_dispose_address=` iamcfg list -d iam -g Dispose -i dispose-address | grep dispose-address `
		RecordCommndInfo "iam" "${cmd_dispose_address}" "${iamcfg_dispose_address}"
	    local dispose_address=$(echo ${iamcfg_dispose_address} | awk -F " " '{print $3}')
	    if [ "${dispose_address}" == "0.0.0.0" ] ||  [[ ! "${dispose_address}" =~ ^([0-9]{1,3}\.){3}[0-9]{1,3}$ ]]
	    then
		    ResultTxtOutput "ERROR" "比对服务地址未配置"
	    else
		    cmd=` ping -c 3 -W 1 "${dispose_address}" 1>/dev/null `
		    if [ $? -ne 0 ]
		    then
		        ResultTxtOutput "ERROR" "比对服务所在服务器网络不通"
		    else 
		        ResultTxtOutput "INFO" "比对服务IP配置正常"
		    fi
	    fi
	
	    # 比对服务模式检查
		local cmd_dispose_type="iamcfg list -d iam -g Dispose -i dispose-type | grep dispose-type"
	    local iamcfg_dispose_type=` iamcfg list -d iam -g Dispose -i dispose-type | grep dispose-type `
		RecordCommndInfo "iam" "${cmd_dispose_type}" "${iamcfg_dispose_type}"
	    local dispose_type=$(echo ${iamcfg_dispose_type} | awk -F " " '{print $3}')
	    if [ "${dispose_type}" == "1" ]
		then
		    LOG 1 "已开启芯片比对服务"
			# 查询是否有比对服务芯片
			local iae_dispose_chip_num=` SelectDB "sqlite" "SELECT COUNT(*) FROM ia_engine WHERE bw_out_ip=\"${dispose_address}\" AND business_type=\"2\"" `
			if [ ${iae_dispose_chip_num} -le 0 ]
			then
			    ResultTxtOutput "ERROR" "已配置比对服务芯片比对，但当前无比对服务芯片"
			else
			    ResultTxtOutput "INFO" "芯片比对配置正常"
			fi
	    elif [ "${dispose_type}" == "0" ]
		then
		    LOG 1 "已开启DBP比对服务"
		else
		    ResultTxtOutput "ERROR" "比对服务类型配置错误"
		fi
	else
	    ResultTxtOutput "INFO" "未开启比对服务"
	fi
}

### MD配置检查
MdCofigCheck()
{
    LOG 1 "开始检查MD配置检查"
    # MD地址检查
	local cmd="iamcfg list -d iam -g Global -i MD-Balance-IP | grep MD-Balance-IP"
    local iamcfg_md_balance_ip=` iamcfg list -d iam -g Global -i MD-Balance-IP | grep MD-Balance-IP `
	RecordCommndInfo "iam" "${cmd}" "${iamcfg_md_balance_ip}"
	local md_balance_ip=$(echo ${iamcfg_md_balance_ip} | awk -F " " '{print $3}')
	if [ "${md_balance_ip}" == "0.0.0.0" ] ||  [[ ! "${md_balance_ip}" =~ ^([0-9]{1,3}\.){3}[0-9]{1,3}$ ]]
	then
		ResultTxtOutput "ERROR" "图片流任务——MD服务器地址未配置"
	else
		cmd=` ping -c 3 -W 1 "${md_balance_ip}" 1>/dev/null `
		if [ $? -ne 0 ]
		then
		    ResultTxtOutput "ERROR" "图片流任务__MD服务器网络不通"
		else 
            MD_BALANCE_IP=${md_balance_ip}
		    ResultTxtOutput "INFO" "图片流任务__MD服务器配置正常"
		fi
	fi
	
	# 结构化视频流转发MD检查
	local cmd_md_forward_enable="iamcfg list -d iam -g VasMdForward -i MD-Forward-Enable | grep MD-Forward-Enable"
	local iamcfg_md_forward_enable=` iamcfg list -d iam -g VasMdForward -i MD-Forward-Enable | grep MD-Forward-Enable `
	RecordCommndInfo "iam" "${cmd_md_forward_enable}" "${iamcfg_md_forward_enable}"
	local md_forward_enable=$(echo ${iamcfg_md_forward_enable} | awk -F " " '{print $3}')
	if [ "${md_forward_enable}" == "0" ]
	then
		ResultTxtOutput "INFO" "机非人视频流任务__未开启MD转发"
	else
	    ResultTxtOutput "INFO" "机非人视频流任务__已开启MD转发"
		# 视频流转发MD地址检查
		local cmd_md_forward_address="iamcfg list -d iam -g Global -i MD-Forward-Address | grep MD-Forward-Address"
	    local iamcfg_md_forward_address=` iamcfg list -d iam -g Global -i MD-Forward-Address | grep MD-Forward-Address `
		RecordCommndInfo "iam" "${cmd_md_forward_address}" "${iamcfg_md_forward_address}"
	    local md_forward_address=$(echo ${iamcfg_md_forward_address} | awk -F " " '{print $3}')
	    if [ "${md_forward_address}" == "0.0.0.0" ] ||  [[ ! "${md_forward_address}" =~ ^([0-9]{1,3}\.){3}[0-9]{1,3}$ ]]
	    then
		    ResultTxtOutput "ERROR" "机非人视频流任务——MD转发服务器地址未配置"
	    else
		    cmd=` ping -c 3 -W 1 "${md_forward_address}" 1>/dev/null `
		    if [ $? -ne 0 ]
		    then
		        ResultTxtOutput "ERROR" "机非人视频流任务——MD转发服务器网络不通"
		    else 
		        ResultTxtOutput "INFO" "机非人视频流任务——MD转发服务器配置正常"
		    fi
	    fi
		
		# 视频流转发模式检查
		local cmd_md_forward_mod="iamcfg list -d iam -g Global -i MD-Forward-Mode | grep MD-Forward-Mode"
		local iamcfg_md_forward_mode=` iamcfg list -d iam -g Global -i MD-Forward-Mode | grep MD-Forward-Mode `
		RecordCommndInfo "iam" "${cmd_md_forward_mod}" "${iamcfg_md_forward_mode}"
	    local md_forward_mode=$(echo ${iamcfg_md_forward_mode} | awk -F " " '{print $3}')
		if [ "${md_forward_mode}" != "2" ]
		then
		    ResultTxtOutput "WARNING" "机非人视频流图片转发模式已开启"
		else
		    ResultTxtOutput "INFO" "机非人视频流URL转发模式已开启"
		fi
	fi
}

### 获取IAM版本号
GetIamVersion()
{
    local iam_version=""
	if [ -f "/usr/local/svconfig/patch/iampatchsys.conf" ]
	then
	    local cmd="displayiapatch.sh | grep PHVERO | grep IAM"
	    local iam_version_extend_info=` displayiapatch.sh | grep PHVERO | grep IAM `
		RecordCommndInfo "iam" "${cmd}" "${iam_version_extend_info}"
		local iam_version_cut_head=${iam_version_extend_info#*-}
		local iam_version=${iam_version_cut_head%_*}
	else
	    local cmd="iamcfgtool.sh -v | grep HCVERO"
	    local iam_version_extend_info=` iamcfgtool.sh -v | grep HCVERO `
		RecordCommndInfo "iam" "${cmd}" "${iam_version_extend_info}"
	    local iam_version_cut_head=${iam_version_extend_info#*-}
	    local iam_version=${iam_version_cut_head%_*}
	fi
	echo "${iam_version}"
}

### DBE IP配置检查
DbeIpCheck()
{
    LOG 1 "开始检查BDE配置"
	local cmd="iamcfg list -d iam -g DBE -i master-address | grep master-address"
    local iamcfg_dbe_master_address=` iamcfg list -d iam -g DBE -i master-address | grep master-address `
	RecordCommndInfo "iam" "${cmd}" "${iamcfg_dbe_master_address}"
	local dbe_master_address=$(echo ${iamcfg_dbe_master_address} | awk -F " " '{print $3}')
	if [ "${dbe_master_address}" == "192.168.0.80" ] ||  [[ ! "${dbe_master_address}" =~ ^([0-9]{1,3}\.){3}[0-9]{1,3}$ ]]
	then
		ResultTxtOutput "ERROR" "DBE存储地址未配置"
	else
		cmd=` ping -c 3 -W 1 "${dbe_master_address}" 1>/dev/null `
		if [ $? -ne 0 ]
		then
		    ResultTxtOutput "ERROR" "DBE存储服务器网络不通"
		else 
		    ResultTxtOutput "INFO" "DBE存储地址配置正常"
		fi
	fi
}

### CDS存储配置检查
CdsStorageCheck()
{
    LOG 1 "开始检查CDS存储配置"
    # vas存储地址检查
	local cmd_cds_adress_vas="iamcfg list -d iam -g DataStorage -i cds-address-vas | grep cds-address-vas"
    local iamcfg_cds_adress_vas=` iamcfg list -d iam -g DataStorage -i cds-address-vas | grep cds-address-vas `
	RecordCommndInfo "iam" "${cmd_cds_adress_vas}" "${iamcfg_cds_adress_vas}"
	local cds_adress_vas=$(echo ${iamcfg_cds_adress_vas} | awk -F " " '{print $3}')
	if [ "${cds_adress_vas}" == "127.0.0.1" ] ||  [[ ! "${cds_adress_vas}" =~ ^([0-9]{1,3}\.){3}[0-9]{1,3}$ ]]
	then
		ResultTxtOutput "WARNING" "机非人CDS存储地址未配置"
	else
		cmd=` ping -c 3 -W 1 "${cds_adress_vas}" 1>/dev/null `
		if [ $? -ne 0 ]
		then
		    ResultTxtOutput "ERROR" "机非人CDS存储服务器网络不通"
		else 
		    ResultTxtOutput "INFO" "机非人CDS存储地址配置正常"
		fi
	fi
	# vas存储目录检查
	local cmd_cds_usercode_vas="iamcfg list -d iam -g DataStorage -i cds-usercode-vas | grep cds-usercode-vas"
	local iamcfg_cds_usercode_vas=` iamcfg list -d iam -g DataStorage -i cds-usercode-vas | grep cds-usercode-vas `
	RecordCommndInfo "iam" "${cmd_cds_usercode_vas}" "${iamcfg_cds_usercode_vas}"
	local cds_usercode_vas=$(echo ${iamcfg_cds_usercode_vas} | awk -F " " '{print $3}')
	if [ "${cds_usercode_vas}" == "CdsPictureDirCode" ] || [[ ! "${cds_usercode_vas}" =~ ^common* ]]
	then
	    ResultTxtOutput "WARNING" "机非人CDS存储目录未配置"
	else
		ResultTxtOutput "INFO" "机非人CDS存储目录已配置"
	fi
	
	# fs存储地址检查
	local cmd_cds_adress_fs="iamcfg list -d iam -g DataStorage -i cds-address-fs | grep cds-address-fs"
    local iamcfg_cds_adress_fs=` iamcfg list -d iam -g DataStorage -i cds-address-fs | grep cds-address-fs `
	RecordCommndInfo "iam" "${cmd_cds_adress_fs}" "${iamcfg_cds_adress_fs}"
	local cds_adress_fs=$(echo ${iamcfg_cds_adress_fs} | awk -F " " '{print $3}')
	if [ "${cds_adress_fs}" == "127.0.0.1" ] || [[ ! "${cds_adress_fs}" =~ ^([0-9]{1,3}\.){3}[0-9]{1,3}$ ]]
	then
		ResultTxtOutput "WARNING" "人脸CDS存储地址未配置"
	else
		cmd=` ping -c 3 -W 1 "${cds_adress_fs}" 1>/dev/null `
		if [ $? -ne 0 ]
		then
		    ResultTxtOutput "ERROR" "人脸CDS存储服务器网络不通"
		else 
		    ResultTxtOutput "INFO" "人脸CDS存储地址配置正常"
		fi
	fi
	
	# fs存储目录检查
	local cmd_cds_usercode_fs="iamcfg list -d iam -g DataStorage -i cds-usercode-fs | grep cds-usercode-fs"
	local iamcfg_cds_usercode_fs=` iamcfg list -d iam -g DataStorage -i cds-usercode-fs | grep cds-usercode-fs `
	RecordCommndInfo "iam" "${cmd_cds_usercode_fs}" "${iamcfg_cds_usercode_fs}"
	local cds_usercode_fs=$(echo ${iamcfg_cds_usercode_fs} | awk -F " " '{print $3}')
	if [ "${cds_usercode_fs}" == "CdsPictureDirCode" ] || [[ ! "${cds_usercode_fs}" =~ ^common ]]
	then
	    ResultTxtOutput "WARNING" "人脸CDS存储目录未配置"
	else
		ResultTxtOutput "INFO" "人脸机非人CDS存储目录已配置"
	fi
	
	# ba存储地址检查
	local cmd_cds_adress_ba="iamcfg list -d iam -g DataStorage -i cds-address-ba | grep cds-address-ba"
    local iamcfg_cds_adress_ba=` iamcfg list -d iam -g DataStorage -i cds-address-ba | grep cds-address-ba `
	RecordCommndInfo "iam" "${cmd_cds_adress_ba}" "${iamcfg_cds_adress_ba}"
	local cds_adress_ba=$(echo ${iamcfg_cds_adress_ba} | awk -F " " '{print $3}')
	if [ "${cds_adress_ba}" == "127.0.0.1" ] || [[ ! "${cds_adress_ba}" =~ ^([0-9]{1,3}\.){3}[0-9]{1,3}$ ]]
	then
		ResultTxtOutput "WARNING" "行为分析CDS存储地址未配置"
	else
		cmd=` ping -c 3 -W 1 "${cds_adress_ba}" 1>/dev/null `
		if [ $? -ne 0 ]
		then
		    ResultTxtOutput "ERROR" "行为分析CDS存储服务器网络不通"
		else 
		    ResultTxtOutput "INFO" "行为分析CDS存储地址配置正常"
		fi
	fi
	
	# ba 存储目录检查
	local cmd_cds_usercode_ba="iamcfg list -d iam -g DataStorage -i cds-usercode-ba | grep cds-usercode-ba"
	local iamcfg_cds_usercode_ba=` iamcfg list -d iam -g DataStorage -i cds-usercode-ba | grep cds-usercode-ba} `
	RecordCommndInfo "iam" "${cmd_cds_usercode_ba}" "${iamcfg_cds_usercode_ba}"
	local cds_usercode_ba=$(echo ${iamcfg_cds_usercode_ba} | awk -F " " '{print $3}')
	if [ "${cds_usercode_ba}" == "CdsPictureDirCode" ] || [[ ! "${cds_usercode_ba}" =~ ^common ]]
	then
	    ResultTxtOutput "WARNING" "行为分析CDS存储目录未配置"
	else
		ResultTxtOutput "INFO" "行为分析CDS存储目录已配置"
	fi
}

### IPSAN存储配置检查
IpsanStorageCheck()
{
    LOG 1 "开始检查IPSAN存储配置"
	#### vas IPSAN存储地址检查
	local cmd_ipsan_forward_ip="iamcfg list -d iam -g VasForward -i forward-ip | grep forward-ip"
	local iamcfg_ipsan_forward_ip=` iamcfg list -d iam -g VasForward -i forward-ip | grep forw `
	RecordCommndInfo "iam" "${cmd_ipsan_forward_ip}" "${iamcfg_ipsan_forward_ip}"
	local ipsan_forward_ip=$(echo ${iamcfg_ipsan_forward_ip} | awk -F " " '{print $3}')
	if [ "${ipsan_forward_ip}" == "0.0.0.0" ] || [[ ! "${ipsan_forward_ip}" =~ ^([0-9]{1,3}\.){3}[0-9]{1,3}$ ]]
	then
	    ResultTxtOutput "ERROR" "机非人IPSAN存储地址未配置"
	else
	    cmd=` ping -c 3 -W 1 "${ipsan_forward_ip}" 1>/dev/null `
		if [ $? -ne 0 ]
		then
		    ResultTxtOutput "ERROR" "机非人IPSAN存储服务器网络不通"
		else
		    ResultTxtOutput "INFO" "机非人IPSAN存储地址配置正常"
		fi
	fi
	
	#### vas IPSAN存储端口检查
	local cmd_ipsan_forward_port="iamcfg list -d iam -g VasForward -i forward-port | grep forward-port"
	local iamcfg_ipsan_forward_port=` iamcfg list -d iam -g VasForward -i forward-port | grep forward-port `
	RecordCommndInfo "iam" "${cmd_ipsan_forward_port}" "${iamcfg_ipsan_forward_port}"
	local ipsan_forward_port=$(echo ${iamcfg_ipsan_forward_port} | awk -F " " '{print $3}')
	if [ "${ipsan_forward_port}" == "0" ] || [[ ! "${ipsan_forward_port}" =~ ^[0-9]*$ ]]
	then
	    ResultTxtOutput "ERROR" "机非人IPSAN存储端口未配置"
	else
	    ResultTxtOutput "INFO" "机非人IPSAN存储端口已配置"
	fi
}

### 存储组网配置检查
StorageNetCheck()
{
    LOG 1 "开始检查存储组网配置"
	local cmd="iamcfg list -d iam -g Global -i project-scope | grep project-scope"
    local iamcfg_project_scope=` iamcfg list -d iam -g Global -i project-scope | grep project-scope `
	RecordCommndInfo "iam" "${cmd}" "${iamcfg_project_scope}"
	local storage_mode=$(echo ${iamcfg_project_scope} | awk -F " " '{print $3}')
	case "${storage_mode}" in
		"small")
			IpsanStorageCheck
			;;
		"big")
			CdsStorageCheck
			;;
		*)
		    ResultTxtOutput "ERROR" "当前配置的存储模式不支持"
	esac
}

### VM数据库密码检查
VmPassworCheck()
{
    LOG 1 "开始检查VM数据库密码配置"
    local iamcfg_pgsql_server=` iamcfg list -d ias -g IMOS -i pgsql-server | grep pgsql-server `
	local iamcfg_pgsql_password=` iamcfg list decrypt -d ias -g IMOS -i pgsql-password | grep pgsql-password `
	local vm_pg_ip=$(echo ${iamcfg_pgsql_server} | awk -F " " '{print $3}')
	local vm_pg_passwd=$(echo ${iamcfg_pgsql_password} | awk -F " " '{print $3}')
    cmd=` ${PGSQL} "host=${vm_pg_ip} port=5432 user=postgres password=${vm_pg_passwd} dbname=imos" -c "SELECT 1" -t > /dev/null 2>&1 `
	if [ $? -ne 0 ]
	then
	    ResultTxtOutput "ERROR" "VM数据库密码配置错误"
	else
	    ResultTxtOutput "INFO" "VM数据库密码配置正常"
	fi
}

### iam服务状态检查
IamServerStatusCheck()
{
    LOG 1 "开始检查IAM服务状态"
    local cmd="iamserver.sh status | grep Service"
    local str_iam_server_status=` iamserver.sh status | grep Service `
	RecordCommndInfo "iam" "${cmd}" "${str_iam_server_status}"
	local process_all_running="true"
	local error_prcess=""
	while read -r iam_server_status_line
	do
	    cmd=` grep Running <<< "${iam_server_status_line}" `
	    if [ $? -ne 0 ]
		then
			if [ -z "${error_prcess}" ];then
				error_prcess=$(echo "${iam_server_status_line}" | awk ' {print $1} ')
				
			else
				local error_process_name=$(echo "${iam_server_status_line}" | awk ' {print $1} ')
				error_prcess="${error_prcess}，${error_process_name}"
			fi
			process_all_running="false"
	    fi
	done <<< "${str_iam_server_status}"
	### iam服务检查状态OK
	if [[ "${process_all_running}" == "true" ]]
	then
		ResultTxtOutput "INFO" "IAM服务状态检查正常"
	else
	    ResultTxtOutput "ERROR" "IAM服务状态存在异常：${error_prcess}"
	fi
}

### iam崩溃检查
IamCoreDumpCheck()
{
    LOG 1 "开始检查IAM近期崩溃情况"
	local cmd="ls /root | grep core | grep IAM"
    local core_file_infos=` ls /root | grep core | grep IAM `
	local check_result="true"
	RecordCommndInfo "iam" "${cmd}" "${core_file_infos}"
	while read -r core_file_line
	do
	     #解析core文件名携带的时间戳。core文件名格式：core_dump-iam-17624-1732513588-11-IAM31
		 local core_dump_time=$(echo ${core_file_line} | awk -F "-" '{print $4}')
		 local current_time=` date +%s `
		 local week=604800
		 let time_diff=current_time-core_dump_time
		 if [ ${time_diff} -lt ${week} ]
		 then
		     check_result="false"
		 else
		     check_result="true"
		 fi
	done <<< "${core_file_infos}"
	if [ "${check_result}" == "true" ]
	then
	    ResultTxtOutput "INFO" "IAM近一周未出现过崩溃"
	else
	    ResultTxtOutput "ERROR" "IAM近一周出现过崩溃"
	fi
}

### iam最低版本检查，低于B3351版本不保证脚本功能正常
### $1 iam版本号，格式：B3351.151.1.H01
IamVersionLimitCheck()
{
    iam_version="$1"
    iam_p_version="${iam_version:1:4}"
    if [ ${iam_p_version} -lt ${VERSION_LIMIT} ]
    then
        ResultTxtOutput "ERROR" "当前软件版本低于最低软件版本限制，当前版本：$1"
        echo "当前版本过低，运行脚本存在未知错误，是否需要继续执行？ y/n"
        while :
        do
            read userChonse
            if [ "${userChonse}" != "y" ]
            then
                if [ "${userChonse}" != "n" ]
                then
                    echo -e "输入错误，请重新输入y或者n"
                    continue;
                fi
                echo "结束运行"
                exit 1
            else
                break;
            fi
        done
    else
        ResultTxtOutput "INFO" "当前软件版本：$1，符合软件最低版本要求"
    fi
}

### IAE的算法分析模式检查
### IAM服务器上面查询配置项即可
CheckAlgoAttrMode()
{
	local cmd="iamcfg list|grep \"algo-attr-mode\" "
    local status=` iamcfg list|grep "algo-attr-mode" `
	RecordCommndInfo "IAM" "${cmd}" "${status}"
    if [ -z "${status}" ];then
        ResultTxtOutput "INFO" "算法分析模式不支持配置，默认为性能模式，部分属性不支持分析"
    else
        # 提取algo-attr-mode的数值
		local mode=$(echo "${status}" | awk 'BEGIN {FS="="} {print $2}' | cut -d' ' -f2)
		if [ "${mode}" == "Part" ];then
			ResultTxtOutput "INFO" "算法分析模式支持配置，当前为性能模式【${mode}】，部分属性不支持分析"
		else
			ResultTxtOutput "INFO" "算法分析模式支持配置，当前为功能模式【${mode}】，属性全部支持分析"
		fi
    fi
}

### IAM pg数据库密码配置检查。密码配置错误，提示是否继续巡检
IamPgPasswordCheck()
{
    iam_pg_passwd_config=` iamcfg list decrypt  -d iam -g Postgres -i pgsql-password | grep pgsql-password`
    iam_pg_passwd_value=$(echo ${iam_pg_passwd_config} | awk -F " " '{print $3}')
    cmd=` ${PGSQL} "host=127.0.0.1 port=${IAM_PG_PORT} user=postgres password=${iam_pg_passwd_value} dbname=ias" -c "SELECT 1" -t > /dev/null 2>&1 `
    if [ $? -ne 0 ]
    then
        ResultTxtOutput "ERROR" "IAM的pg数据库密码配置错误"
    else
        ResultTxtOutput "INFO" "IAM的pg数据库密码配置正常"
    fi
    IAM_PG_PASSWD="${iam_pg_passwd_value}"
}

### 行为分析报警阈值（ba-alarm-data-save-threshold）检查
BaAlarmDataSaveThresholdCheck()
{
	local cmd="iamcfg list -d iam -g Global -i ba-alarm-data-save-threshold | grep ba-alarm-data-save-threshold"
	ba_alarm_threshold_config=` iamcfg list -d iam -g Global -i ba-alarm-data-save-threshold | grep ba-alarm-data-save-threshold `
	RecordCommndInfo "iam" "${cmd}" "${ba_alarm_threshold_config}"
    
	ba_alarm_threshold=$(echo ${ba_alarm_threshold_config} | awk -F " " '{print $3}')
    if [ 0 -le ${ba_alarm_threshold} ] && [ 100000000 -ge ${ba_alarm_threshold} ]
	then
		ResultTxtOutput "INFO" "行为分析告警阈值设置正常"
	else
		ResultTxtOutput "ERROR" "行为分析告警阈值超出0-100000000的范围, 当前阈值：${ba_alarm_threshold}"
	fi
}	

### 业务检查入口
IamBussinessCheck()
{
    # iam服务运行状态
	StartCheckItemJsonOutput "${CHECK_SERVER_STATUS}" #巡检头
    IamServerStatusCheck 
    IAMUpdateProgress
	EndCheckItemJsonOutput #结束检查项
	
    # iam崩溃检查
	StartCheckItemJsonOutput "${CHECK_ENV_ITEM_CORE}" #巡检头
    IamCoreDumpCheck
    IAMUpdateProgress
	EndCheckItemJsonOutput #结束检查项
	
    # vm数据库密码检查
	StartCheckItemJsonOutput "${CHECK_VM_DB_PASSWORD}" #巡检头
    VmPassworCheck
    IAMUpdateProgress
	EndCheckItemJsonOutput #结束检查项
	
    # 存储组网检查
	StartCheckItemJsonOutput "${CHECK_CDS_IPSAN_CONFIG}" #巡检头
    StorageNetCheck
    IAMUpdateProgress
	EndCheckItemJsonOutput #结束检查项
	
    # DBE ip配置检查
	StartCheckItemJsonOutput "${CHECK_DBE_IP_STATUS}" #巡检头
    DbeIpCheck
    IAMUpdateProgress
	EndCheckItemJsonOutput #结束检查项
	
    # MD相关配置检查
	StartCheckItemJsonOutput "${CHECK_MD_IP_STATUS}" #巡检头
    MdCofigCheck
    IAMUpdateProgress
	EndCheckItemJsonOutput #结束检查项
	
    # 布控组网检查
	StartCheckItemJsonOutput "${CHECK_DISPOSE_CONFIG}" #巡检头
    DisposeNetCheck
    IAMUpdateProgress
	EndCheckItemJsonOutput #结束检查项
	
    # 人脸静态导库组网检查
	StartCheckItemJsonOutput "${CHECK_STATICFACE_CONFIG}" #巡检头
    StaticFaceEngineTypeCheck
    IAMUpdateProgress
	EndCheckItemJsonOutput #结束检查项
	
	#分析模式校验
	StartCheckItemJsonOutput "${CHECK_ALGO_ATTR_MODE}" #巡检头
    CheckAlgoAttrMode
    IAMUpdateProgress
	EndCheckItemJsonOutput #结束检查项

	# 芯片状态检查
	StartCheckItemJsonOutput "${CHECK_CHIP_RUN_STATUS}" #巡检头
    ChipRunStatusCheck
    IAMUpdateProgress
	EndCheckItemJsonOutput #结束检查项
	
	# IAM防火墙状态检查
    StartCheckItemJsonOutput "${CHECK_IAM_FIREWALLD_STATUS}" #巡检头
    FirewalldStatusCheck
    IAMUpdateProgress
    EndCheckItemJsonOutput #结束检查项
    
    # 人脸图片流负载均衡状态检查
    StartCheckItemJsonOutput "${CHECK_FACE_BALANCE_STATUS}" #巡检头
    FacePicBalanceStatusCheck
    IAMUpdateProgress
    EndCheckItemJsonOutput #结束检查项
	
	# 芯片资源分配检查
	StartCheckItemJsonOutput "${CHECK_ASSIGN_RESOURCE}" #开始检查项JSON输出
	result=` python ${ASSIGN_RESOURCE_CHECK_FILE} ` # 调用python脚本进行巡检
	FormatPyhtonCheckReuslt $? "${result}" # 记录python脚本检查结果
	IAMUpdateProgress
	EndCheckItemJsonOutput #结束检查项
	
	# 授权检查
	StartCheckItemJsonOutput "${CHECK_AUTHORIZE}" #开始检查项JSON输出
	result=` python ${AUTHORIZE_CHECK_FILE} ` # 调用python脚本进行巡检
	FormatPyhtonCheckReuslt $? "${result}" # 记录python脚本检查结果
	IAMUpdateProgress
	EndCheckItemJsonOutput #结束检查项
	
	# 资源分配和授权匹配检查
	StartCheckItemJsonOutput "${CHECK_ASSIGN_AND_AUTHORIZE}" #开始检查项JSON输出
	result=` python ${ASSIGN_AND_AUTHORIZE_CHECK_FILE} ` # 调用python脚本进行巡检
	FormatPyhtonCheckReuslt $? "${result}" # 记录python脚本检查结果
	IAMUpdateProgress
	EndCheckItemJsonOutput #结束检查项
	
	# 授权时间检查
	StartCheckItemJsonOutput "${CHECK_AUTHORIZE_TIME}" #开始检查项JSON输出
	result=` python ${AUTHORIZE_TIME_CHECK_FILE} ` # 调用python脚本进行巡检
	FormatPyhtonCheckReuslt $? "${result}" # 记录python脚本检查结果
	IAMUpdateProgress
	EndCheckItemJsonOutput #结束检查项
	
	#资源分配和任务数巡检
	StartCheckItemJsonOutput "${CHECK_TASK_RESOURCE_MATCH}" #开始检查项JSON输出
	#入参
	dbname="ias"
	user="postgres"
	password="${IAM_PG_PASSWD}"
	host="127.0.0.1"
	port=${IAM_PG_PORT}
    result=` python ${TASK_RESOURCE_MATCH_CHECK_FILE} ${dbname} ${user} ${password} ${host} ${port} ` # 调用python脚本进行巡检
    FormatPyhtonCheckReuslt $? "${result}" # 记录python脚本检查结果
    IAMUpdateProgress
    EndCheckItemJsonOutput #结束检查项JSON输出

	# 行为分析报警阈值（ba-alarm-data-save-threshold）检查
	StartCheckItemJsonOutput "${CHECK_BA_ALARM_THRESHOLD}" #巡检头
	BaAlarmDataSaveThresholdCheck
	IAMUpdateProgress
	EndCheckItemJsonOutput #结束检查项
}