#!/bin/bash

#缓存检查项的结果，并输出json

# 声明关联数组，用于存储检测项目及其对应的字符串值
declare -A detection_project_map

# 声明关联数组，用于存储检测项目的结果（ok 或 ng）
declare -A result_map

# 将字符串值添加到指定的检测项目中
# 参数:
#   $1 - 检测项目名称（键）
#   $2, $3, ... - 字符串值
function add_string_to_detection_project() {
    local project_name="$1"  # 获取第一个参数作为检测项目名称
    shift                    # 移除第一个参数，剩余的参数作为新字符串值
    local new_strings=("$@")  # 将剩余的参数存储到new_strings数组中

    if [[ -n "${detection_project_map[$project_name]}" ]]; then
        # 如果检测项目已经存在，获取现有的字符串值并转换为数组
        IFS=' ' read -r -a existing_strings <<< "${detection_project_map[$project_name]}"
        # 将新字符串值追加到现有字符串值中
        strings=("${existing_strings[@]}" "${new_strings[@]}")
    else
        # 如果检测项目不存在，直接使用新字符串值
        strings=("${new_strings[@]}")
    fi

    # 将更新后的字符串数组内容作为一个字符串存储到detection_project_map中
    detection_project_map["$project_name"]="${strings[*]}"
}

# 设置检测项目的状态（ok 或 ng）
# 参数:
#   $1 - 检测项目名称（键）
#   $2 - 状态值（1 表示 ok，0 表示 ng）
function set_detection_project_status() {
    local project_name="$1"  # 获取第一个参数作为检测项目名称
    local status="$2"        # 获取第二个参数作为状态值

	# 检查result_map中对应的项目是否为ng
    if [[ "${result_map[$project_name]}" == "ng" ]]; then
        echo "Detection Project: $project_name already marked as ng. Skipping further additions."
        return
    fi

    if [[ "$status" -eq 1 ]]; then
        result_map["$project_name"]="ok"
    elif [[ "$status" -eq 0 ]]; then
        result_map["$project_name"]="ng"
    else
        echo "Invalid status value. Use 1 for ok or 0 for ng."
    fi
}

# 从检测项目中获取字符串值
# 参数:
#   $1 - 检测项目名称（键）
function get_strings_from_detection_project() {
    local project_name="$1"  # 获取第一个参数作为检测项目名称
    local strings_str="${detection_project_map[$project_name]}"  # 从detection_project_map中获取对应的字符串值

    if [[ -n "$strings_str" ]]; then
        IFS=' ' read -r -a strings <<< "$strings_str"  # 将字符串转换回数组
        echo "Detection Project: $project_name"
        for string in "${strings[@]}"; do
            # 输出字符串
            echo "  String: $string"
        done
    else
        echo "Detection Project: $project_name not found"
    fi
}

# 从检测项目中获取状态值
# 参数:
#   $1 - 检测项目名称（键）
function get_detection_project_status() {
    local project_name="$1"  # 获取第一个参数作为检测项目名称
    local status="${result_map[$project_name]}"  # 从result_map中获取对应的状态值

    if [[ -n "$status" ]]; then
        echo "Detection Project: $project_name, Status: $status"
    else
        echo "Detection Project: $project_name not found in result_map"
    fi
}

# 汇总并打印所有检测项目的JSON格式
# 参数:
#   $1 - 是否最后一个报告 1表示是最后，其余表示不是。
function print_detection_projects_json() {
    local json_output="["
    local first_entry=true

    for project_name in "${!detection_project_map[@]}"; do
        local check_msg="${detection_project_map[$project_name]}"
        local check_result="${result_map[$project_name]}"

        # 如果result_map中没有对应的项目，则默认设置为"unknown"
        if [[ -z "$check_result" ]]; then
            check_result="unknown"
        fi

        # 将字符串转换为JSON数组格式
        IFS=' ' read -r -a strings <<< "$check_msg"
        local check_msg_json="["
        local first_string=true
        for string in "${strings[@]}"; do
            if [ "$first_string" = true ]; then
                first_string=false
            else
                check_msg_json+=", "
            fi
            check_msg_json+="\"$string\""
        done
        check_msg_json+="]"

        # 构建JSON对象
        if [ "$first_entry" = true ]; then
            first_entry=false
        else
            json_output+=", "
        fi
        json_output+="{\"ItemName\":\"$project_name\", \"CheckMsg\":$check_msg_json, \"CheckResult\":\"$check_result\"}"
    done
	if [[ "$1" -eq 1 ]]; then
         json_output+="]"
    else
		json_output+="],"
    fi
    echo "$json_output"
}

# 示例：添加字符串到检测项目
add_string_to_detection_project "网络连通性检查" "206.206.124.186:10245端口监听正常" "206.206.124.210:10245端口监听正常"
add_string_to_detection_project "IAM端口检查" "IAM:10240端口监听正常" "IAM:5832端口监听正常" "IAM:5347端口监听正常" "IAM:30320端口监听正常"

# 示例：设置检测项目的状态
set_detection_project_status "网络连通性检查" 1  # 设置为ok
set_detection_project_status "IAM端口检查" 0     # 设置为ng

# 示例：添加字符串到检测项目（在设置为ng后尝试添加）
add_string_to_detection_project "IAM端口检查" "IAM:30321端口监听正常"  # 应该被跳过

# 示例：汇总并打印所有检测项目的JSON格式
print_detection_projects_json
