#!/bin/bash

##替换库检查，验证是否存在替换库，替换库是否具备执行权限

#替换库时间戳阈值90分钟
LIB_TIME_THRESHOLD=5400 #（秒数）

# 定义一个函数来检查文件时间戳
### $1 基准时间，通过产品化脚本iaecfgtool.sh -v获取的大版本时间戳   例如：2023-09-20@03:13
### $2 基准时间，通过产品化脚本displayiapatch.sh -v获取的补丁时间戳 例如：2024-12-13 14:14
### $3 是否校验文件权限  只校验bin目录下的文件，替换库文件不进行校验  1进行校验 0不校验
### $4 文件列表信息：文件名 文件修改时间戳 文件权限（例如：/opt/wise_iae/ia12k/sbin/iaeserver.py 1695149265 755）
### $5 日志前缀信息，包括【IP】【引擎名称】【芯片号】
check_file_timestamps() 
{
    # 第一个参数是基准时间1
    local build_time1="$1"
    
    # 将BUILDTIME1转换为时间戳（秒）
    local build_time1_epoch=$(date -d "$1" +%s)
    
    # 第二个参数是基准时间2（可能为空）
    local build_time2="$2"
    
    # 将BUILDTIME2转换为时间戳（秒），如果为空则为0
    local build_time2_epoch=0
    if [ -n "$build_time2" ]; then
        build_time2_epoch=$(date -d "$2" +%s)
    fi
    
    # 第三个参数是文件列表，格式为 "filename1 timestamp1 permission1 filename2 timestamp2 permission2 ..."
    local file_list="$4"
    
    # 将文件列表按行分割
    while read -r line; do
        # 提取文件名和时间戳
        local file_name=$(echo "$line" | awk '{print $1}')
        local file_time=$(echo "$line" | awk '{print $2}')
		local file_permissions=$(echo "$line" | awk '{print $3}')
        
		# 格式化文件时间为YYYY-MM-DD HH:MM
		local file_format_time=$(date -d "@$file_time" +"%Y-%m-%d %H:%M")

        # 计算时间差1
        local time_diff1=$((build_time1_epoch - file_time))
		if [ $time_diff1 -lt 0 ]; then
            time_diff1=$((time_diff1 * -1))
        fi
        
        # 计算时间差2
        local time_diff2=0
        if [ $build_time2_epoch -ne 0 ]; then
            time_diff2=$((build_time2_epoch - file_time))
			 if [ $time_diff2 -lt 0 ]; then
                time_diff2=$((time_diff2 * -1))
            fi
        fi
        
		# 判读时间戳是否存在问题
		# 1、没有安装补丁的情况下，如果时间差1大于15分钟，表示文件不在基线库包中，错误提示
		# 2、安装补丁的情况下，需要判断时间差1和时间差2同时大于15分钟，表示文件不在补丁包和基线库包中，进行错误提示
		if [ $build_time2_epoch -eq 0 ];then  #未安装补丁
			if [ $time_diff1 -gt $LIB_TIME_THRESHOLD ];then
				if [ $3 -eq 1 ];then #进行替换库权限校验
					if [[ ! $file_permissions =~ [1357]$ ]]; then  #检查文件的权限字符串（%a）的最后一位是否包含 1、3、5 或 7，这些值表示文件具有可执行权限。如果不包含这些值，则输出警告信息。
						ResultTxtOutput "ERROR" "$5：文件【$file_name】存在替换，文件最后修改时间【$file_format_time】比版本时间【$1】相差超过15分钟，并且没有可执行权限"
					fi
				else
					ResultTxtOutput "ERROR" "$5：文件【$file_name】存在替换，文件最后修改时间【$file_format_time】比版本时间【$1】相差超过15分钟"
				fi
			else
				ResultTxtOutput "INFO" "$5：文件【$file_name】不存在替换，文件最后修改时间【$file_format_time】和版本时间【$1】相近"
			fi
		else #安装补丁
			if [ $time_diff1 -gt $LIB_TIME_THRESHOLD ] && [ $time_diff2 -gt $LIB_TIME_THRESHOLD ];then
				if [ $3 -eq 1 ];then #进行替换库权限校验
					if [[ ! $file_permissions =~ [1357]$ ]]; then  #检查文件的权限字符串（%a）的最后一位是否包含 1、3、5 或 7，这些值表示文件具有可执行权限。如果不包含这些值，则输出警告信息。
						ResultTxtOutput "ERROR" "$5：文件【$file_name】存在替换，文件最后修改时间【$file_format_time】比版本时间【$1】和补丁时间【$2】相差超过15分钟，并且没有可执行权限"
					else
						ResultTxtOutput "ERROR" "$5：文件【$file_name】存在替换，文件最后修改时间【$file_format_time】比版本时间【$1】和补丁时间【$2】相差超过15分钟，存在可执行权限"
					fi
				else
					ResultTxtOutput "ERROR" "$5：文件【$file_name】存在替换，文件最后修改时间【$file_format_time】比版本时间【$1】和补丁时间【$2】相差超过15分钟"
				fi
			else
				ResultTxtOutput "INFO" "$5：文件【$file_name】的修改时间【$file_format_time】和版本时间【$1】或者补丁时间【$2】相近"
			fi
		fi
    done <<< "$file_list"
}




### IAM替换库校验和权限校验
Check_IAM_File_Time()
{
	#获取基线版本信息
	local base_time=$(2>/dev/null iamcfgtool.sh -v|grep "BUILDTIME"|awk ' BEGIN {FS="="} {print $2}'| tr '@' ' ' ) 
	RecordCommndInfo "IAM" " iamcfgtool.sh -v|grep \"BUILDTIME\"|awk ' BEGIN {FS=\"=\"} {print $2}' "  "${base_time}"
	if [ -z "${base_time}" ]; then
		ResultTxtOutput "WARNING" "IAM:iamcfgtool.sh -v命令返回为空，校验IAM替换库失败"
		return
	fi
	#获取补丁版本信息
	local patch_time=$( 2>/dev/null displayiapatch.sh -v |grep "IAM" | grep "BUILDTIME" |awk ' BEGIN {FS="="} {print $2}')
	RecordCommndInfo "IAM" " displayiapatch.sh -v |grep \"IAM\" | grep \"BUILDTIME\"|awk ' BEGIN {FS=\"=\"} {print $2}' "  "${patch_time}"
	if [ -z "${patch_time}" ]; then
		LOG 1 "IAM:displayiapatch.sh -v命令返回为空"
	fi
	#获取bin目录中的文件列表
	local bin_flies=$(2>/dev/null find "/opt/wise/ia12k/bin" -type f ! -name '*.*' -exec stat -c "%n %Y %a" {} + )
	RecordCommndInfo "IAM" " find \"/opt/wise/ia12k/bin/\" -type f -exec stat -c \"%n %Y %a\" {} + "  "${bin_flies}"
	#获取lib目录下的文件列表(IAM占时不校验)
	
	check_file_timestamps "$base_time" "$patch_time" 1 "$bin_flies" "【IAM】"
}

### BW的替换库检查
### $1 板卡ip
### $2 板卡ssh端口
### $3 板卡ssh用户名
### $4 板卡ssh密码 
Check_BW_File_Time()
{
	#获取基线版本信息
	local base_verson_info=""
	#根据IP获取板卡名称
	local cmd=""
	local engine_name=$(Get_Engine_Name $1)
	if [ "$engine_name" == "昆仑2" ];then
		cmd="/program/bin/bwcfgtool.sh -v"
	else
		cmd="bwcfgtool.sh -v"
	fi
	base_verson_info=$(SshCmd $1 $2 $3 $4 "$cmd" )
	RecordCommndInfo "$1" "$cmd"  "${base_verson_info}"
	if [ -z "${base_verson_info}" ]; then
		ResultTxtOutput "WARNING" "【$1】bw执行:$cmd命令返回为空,校验BW替换库失败"
		return
	fi
	local base_time=$(2>/dev/null echo "${base_verson_info}"|grep "BUILDTIME"|awk ' BEGIN {FS="="} {print $2}' | tr '@' ' ')
	#获取补丁版本信息
	if [ "$engine_name" == "昆仑2" ];then
		cmd="/program/bin/displayiapatch.sh -v"
	else
		cmd="displayiapatch.sh -v"
	fi
	local patch_verson_info=$(SshCmd $1 $2 $3 $4 "$cmd")
	RecordCommndInfo "$1" "$cmd"  "${patch_verson_info}"
	if [ -z "${patch_verson_info}" ]; then
		LOG 1 "【$1】bw执行:$cmd命令返回为空"
	fi
	local patch_time=$( 2>/dev/null echo "${patch_verson_info}" | grep "IA[0-9]" | grep "BUILDTIME"|awk ' BEGIN {FS="="} {print $2}')
	#获取bin目录中的文件列表
	local get_bin_files_cmd="find \"/opt/wise_proxy/ia12k/bin\" -type f ! -name '*.*' -exec stat -c \"%n %Y %a\" {} +"
	local bin_flies=$(SshCmd $1 $2 $3 $4 "${get_bin_files_cmd}" 2>/dev/null)
	RecordCommndInfo "$1" "${get_bin_files_cmd}"  "${bin_flies}"
	if [ -z "${bin_flies}" ]; then
		ResultTxtOutput "WARNING" "【$1】bw执行:${get_bin_files_cmd}命令返回为空,校验BW替换库失败"
		return
	fi
	#获取lib目录下的文件列表(BW占时不校验)
	
	check_file_timestamps "$base_time" "$patch_time" 1 "$bin_flies" "【$1】【BW】"
}



### IAE的替换库检查
### $1 板卡ip
### $2 板卡ssh端口
### $3 板卡ssh用户名
### $4 板卡ssh密码 
Check_IAE_File_Time()
{
	#根据IP获取板卡名称
	local engine_name=$(Get_Engine_Name $1)

	# 获取板卡名称和芯片个数
	local chip_num=$(Get_Chip_Num $1)
	
	#从主控上面获取下面对应的信息
	#获取基线版本信息
	local base_verson_info=""
	local cmd=""
	if [ "$engine_name" == "昆仑2" ];then
		cmd="/program/bin/iaecfgtool.sh -v"
	else
		cmd="iaecfgtool.sh -v"
	fi
	base_verson_info=$(SshCmd $1 $2 $3 $4 "$cmd" )
	RecordCommndInfo "$1" "$cmd"  "${base_verson_info}"
	if [ -z "${base_verson_info}" ]; then
		ResultTxtOutput "WARNING" "${1}:bwcfgtool.sh -v命令返回为空,校验BW替换库失败"
		return
	fi
	local base_time=$(2>/dev/null echo "${base_verson_info}"|grep "BUILDTIME"|awk ' BEGIN {FS="="} {print $2}'| tr '@' ' ')
	#获取补丁版本信息
	local patch_verson_info=""
	cmd=""
	if [ "$engine_name" == "昆仑2" ];then
		cmd="/program/bin/displayiapatch.sh -v"
	else
		cmd="displayiapatch.sh -v"
		
	fi
	patch_verson_info=$(SshCmd $1 $2 $3 $4 "$cmd" )
	RecordCommndInfo "$1" "$cmd"  "${patch_verson_info}"
	if [ -z "${patch_verson_info}" ]; then
		LOG 1 "$1:displayiapatch.sh -v命令返回为空"
	fi
	local patch_time=$(2>/dev/null echo "${patch_verson_info}" | grep "IA[0-9]" | grep "BUILDTIME"|awk ' BEGIN {FS="="} {print $2}' )
	
	#获取对应的文件
	local index=0
	for ((; index < chip_num; index++))
	do
		# GPU、爱芯、寒武纪、昆仑3只需要检查一次
		if [ "$engine_name" == "GPU" ] || [ "$engine_name" == "AX" ] || [ "$engine_name" == "昆仑3" ] || [ "$engine_name" == "寒武纪" ];then
			if [ "$index" -ne 0 ];then
				return 0
			fi
		fi
		#获取bin文件信息
		local get_bin_files_cmd=$(Get_Bin_Files_Command "$engine_name" "$index" "$1")
		local cmd_ret=` SshCmd $1 $2 $3 $4 "$get_bin_files_cmd" 2>/dev/null `
		RecordCommndInfo "$1" "${get_bin_files_cmd}" "${cmd_ret}"
		if [ -z "${cmd_ret}" ]; then
			if [ "$engine_name" == "昆仑2" ];then
				ResultTxtOutput "WARNING" "${engine_name}:${1}:${index}号芯片:${get_bin_files_cmd}命令返回为空"
			else
				ResultTxtOutput "WARNING" "${engine_name}:${1}:${get_bin_files_cmd}命令返回为空"
			fi
		else
			if [ "$engine_name" == "昆仑2" ];then
				check_file_timestamps "$base_time" "$patch_time" 1 "$cmd_ret" "【$1】【${engine_name}】【${index}号芯片】"
			else
				check_file_timestamps "$base_time" "$patch_time" 1 "$cmd_ret" "【$1】【${engine_name}】"
			fi
		fi
		
		
		#获取lib文件信息
		local get_lib_files_cmd=$(Get_Lib_Files_Command "$engine_name" "$index" "$1")
		cmd_ret=` SshCmd $1 $2 $3 $4 "$get_lib_files_cmd" 2>/dev/null `
		RecordCommndInfo "$1" "${get_lib_files_cmd}" "${cmd_ret}"
		if [ -z "${cmd_ret}" ]; then
			if [ "$engine_name" == "昆仑2" ];then
				LOG 1 "${engine_name}:${1}:${index}号芯片:${get_lib_files_cmd}命令返回为空"
			else
				LOG 1 "${engine_name}:${1}:${get_lib_files_cmd}命令返回为空"
			fi
		else
			if [ "$engine_name" == "昆仑2" ];then
				check_file_timestamps "$base_time" "$patch_time" 0 "$cmd_ret" "【$1】【${engine_name}】【${index}号芯片】"
			else
				check_file_timestamps "$base_time" "$patch_time" 0 "$cmd_ret" "【$1】【${engine_name}】"
			fi
		fi
	done
}


### 获取基线文件列表
### $1板卡名称
### $2芯片编号
### $3板卡IP
Get_Bin_Files_Command()
{
    local command=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "GPU")
        command="find \"/opt/wise_iae/ia12k/process/bin\" -type f ! -name '*.*' -exec stat -c \"%n %Y %a\" {} +"
        ;;
    "AX")
        command="find \"/opt/wise_iae/ia12k/process/bin\" -type f ! -name '*.*' -exec stat -c \"%n %Y %a\" {} +"
        ;;
    "寒武纪")
        command="find \"/opt/wise_iae/ia12k/process/bin\" -type f ! -name '*.*' -exec stat -c \"%n %Y %a\" {} +"
        ;;
    "昆仑3")
        command="find \"/opt/wise_iae/ia12k/process/bin\" -type f ! -name '*.*' -exec stat -c \"%n %Y %a\" {} +"
        ;;
    "昆仑2")
	    command=" ssh ${BOARD_CHIP_IP[$3-$2]} \" find \"/opt/wise/ia12k/bin\" -type f ! -name '*.*' -exec stat -c '%n %Y %a' {} + \" "
	    ;;
     *)
        ResultTxtOutput "WARNING" "不支持的芯片类型：$1"
        ;;
   esac
   echo ${command}
 }
 
 
 
### 获取lib目录下的文件列表
### $1板卡名称
### $2芯片编号
### $3板卡IP
Get_Lib_Files_Command()
{
    local command=""
    ipLastIndex=$((($2+1)*2-1))
    case $1 in
    "GPU")
        command="find \"/opt/wise_iae/ia12k/process/lib\" -type f \( -name 'libalgodata*' -o -name 'libunvalgo*' \) -exec stat -c \"%n %Y %a\" {} +"
        ;;
    "AX")
        command="find \"/opt/wise_iae/ia12k/process/lib\" -type f \( -name 'libalgodata*' -o -name 'libunvalgo*' \) -exec stat -c \"%n %Y %a\" {} +"
        ;;
    "寒武纪")
        command="find \"/opt/wise_iae/ia12k/process/lib\" -type f \( -name 'libalgodata*' -o -name 'libunvalgo*' \) -exec stat -c \"%n %Y %a\" {} +"
        ;;
    "昆仑3")
        command="find \"/opt/wise_iae/ia12k/process/lib\" -type f \( -name 'libalgodata*' -o -name 'libunvalgo*' \) -exec stat -c \"%n %Y %a\" {} +"
        ;;
    "昆仑2")
	    command=" ssh ${BOARD_CHIP_IP[$3-$2]} \"find \"/opt/wise/ia12k/lib\" -type f \( -name 'libalgodata*' -o -name 'libunvalgo*' \) -exec stat -c '%n %Y %a' {} + \" "
	    ;;
     *)
        ResultTxtOutput "WARNING" "不支持的芯片类型：$1"
        ;;
   esac
   echo ${command}
 }
 
 